:- use_module(library(dyLog)).

:- use_module(db).

:- op(200,xfx,['!=']).

X '!=' Y :- X \== Y.



% KB on Computer

component(computer(multimedia),peripheral(modem)).

component(computer(multimedia),peripheral(audio)).

component(computer(multimedia),monitor('19inch')).

component(computer(multimedia),ram(generic)).

component(computer(multimedia),cpu(generic)).

component(computer(multimedia),mother(generic)).







component(computer(cad),peripheral(plotter)).

component(computer(cad),monitor('19inch')).

component(computer(cad),ram('256Mb')).

component(computer(cad),cpu('pIII800')).

component(computer(cad),mother('asusp3')).



component(computer(wordprocessing),peripheral(printer)).

component(computer(wordprocessing),monitor(generic)).

component(computer(wordprocessing),ram(generic)).

component(computer(wordprocessing),cpu(generic)).



% Dipendencies among the components



dependency(cpu(_X),mother(generic)).





%functional fluents



functionalFluent(computer/1).

functionalFluent(type_monitor/1).

functionalFluent(type_ram/1).

functionalFluent(type_cpu/1).

functionalFluent(credit/1).



%initial observations



obs(credit(0)).



%fluent domains



computer(X) domain (X in [cad,multimedia,wordprocessing]). 

budget(X) domain (X in [450,550,650,750,850]).



type_monitor(X) domain (db:get_article(category(monitor),X)).

type_ram(X) domain (db:get_article(category(ram),X)).

type_cpu(X) domain (db:get_article(category(cpu),X)).

peripheral(X) domain (db:get_article(category(peripheral),X)).

requested(X) domain (X in [computer(cad),computer(multimedia),computer(wordprocessing)]).

 

%the seller



build_a_computer(Plan) isp

        get_user_preferences &

        ask_max_value_budget &	

	+plan(?credit(C) & ?budget(B) & +(C =< B) after assembly,Plan).





get_user_preferences isp

        offer_computer_type &

	?goal(has(ram(R))) & ask_has_ram(R) &

	?goal(has(cpu(C))) & ask_has_cpu(C) &

	?goal(has(monitor(M))) & ask_has_monitor(M).



offer_computer_type possible_if true.

offer_computer_type suggests requested(_X).

suggests_routine(_Domain,requested,CompAppl):-

	format("What do you need the computer for?\n",[]),read(CompAppl).

	

ask_has_ram(X) possible_if ?u(user_has(ram(X))).

ask_has_ram(X) senses user_has(ram(X)).

senses_routine(user_has(ram(Type)),Bool):- format("Do you already have ram  ~p? ",[Type]),read(Bool).



ask_has_cpu(X) possible_if ?u(user_has(cpu(X))).

ask_has_cpu(X) senses user_has(cpu(X)).

senses_routine(user_has(cpu(Type)),Bool):- format("Do you already have cpu  ~p? ",[Type]),read(Bool).



ask_has_monitor(X) possible_if ?u(user_has(monitor(X))).

ask_has_monitor(X) senses user_has(monitor(X)).

senses_routine(user_has(monitor(Type)),Bool):- format("Do you already have monitor  ~p? ",[Type]),read(Bool).





ask_max_value_budget possible_if true.

ask_max_value_budget senses budget(_B).

senses_routine(budget,V):-

	format("Which maximum budget do you have? {450,550,650,750,850}: ",[]),

	read(V).





%causal rules



has(computer(cad)) if

   ?has(mother(asusp3)) & ?has(peripheral(plotter)) & ?has(monitor('19inch')) & ?has(ram('256Mb')) & ?has(cpu('pIII800')).

has(computer(multimedia)) if

   ?has(mother(generic)) & ?has(peripheral(modem)) & ?has(peripheral(audio)) & ?has(monitor('19inch')) & ?has(ram(generic)) & ?has(cpu(generic)).

has(computer(wordprocessing)) if

   ?has(mother(generic)) & ?has(peripheral(printer)) & ?has(monitor(generic)) & ?has(ram(generic)) & ?has(cpu(generic)).



											

goal(has(X)) if ?requested(X) & ?u(has(X)).

goal(has(X)) if ?requested(X) & ?(~(has(X))).

goal(has(C)) if ?goal(has(computer(X))) & +component(computer(X),C).





user_has(Y) if ?user_has(X) & +dependency(X,Y).

has(X) if ?user_has(X).



~goal(has(X)) if ?has(X).

has(mother(generic)) if ?has(mother(_)).

has(cpu(generic)) if ?has(cpu(_)).

has(ram(generic)) if ?has(ram(_)).

has(monitor(generic)) if ?has(monitor(_)).



~goal(has(X)) if ?has(X).

assembled if ?has(computer(_)).

~assembled if ?goal(_X).



%assemble



assembly isp ?(assembled).

assembly isp ?(~assembled) & achieve_goal & assembly.



achieve_goal isp ?goal(has(monitor(generic))) &  offer_monitor_type & ?type_monitor(X) & add(monitor(X)).

achieve_goal isp ?goal(has(monitor(X))) & +(X '!=' generic) & add(monitor(X)).



achieve_goal isp ?goal(has(ram(generic))) & offer_ram_type & ?type_ram(X) & add(ram(X)).

achieve_goal isp ?goal(has(ram(X))) & +(X '!=' generic) & add(ram(X)).



achieve_goal isp ?goal(has(cpu(generic))) & offer_cpu_type & ?type_cpu(X) & add(cpu(X)).

achieve_goal isp ?goal(has(cpu(X))) & +(X '!=' generic) & add(cpu(X)).



achieve_goal isp ?goal(has(peripheral(X))) & add(peripheral(X)).



achieve_goal isp ?goal(has(mother(X))) & add(mother(X)).





offer_monitor_type possible_if true. 

offer_monitor_type suggests type_monitor(_X).

suggests_routine(Domain,type_monitor,V):- format("\n Domain ~p.\n Select:",[Domain]),read(V).



offer_ram_type possible_if true. 

offer_ram_type suggests type_ram(_X).

suggests_routine(Domain,type_ram,V):- format("\n Domain ~p.\n Select:",[Domain]),read(V).



offer_cpu_type possible_if true. 

offer_cpu_type suggests type_cpu(_X).

suggests_routine(Domain,type_cpu,V):- format("\n Domain ~p.\n Select:",[Domain]),read(V).



% Atomic actions



add(monitor(_X)) possible_if true.

add(monitor(X)) causes has(monitor(X)) if true.

add(monitor(X)) causes credit(B1) if +get_value(X,price,P) & ?credit(B) & +(B1 is B + P).



add(ram(_X)) possible_if true.

add(ram(X)) causes has(ram(X)) if true.



add(ram(X)) causes credit(B1) if +get_value(X,price,P) & ?credit(B) & +(B1 is B + P).



add(cpu(_X)) possible_if true.

add(cpu(X)) causes has(cpu(X)) if true.

add(cpu(X)) causes credit(B1) if +get_value(X,price,P) & ?credit(B) & +(B1 is B + P).









add(mother(_X)) possible_if ?has(cpu(_G)).

add(mother(generic)) causes has(mother(Y)) if ?has(cpu(Cpu)) & +get_mother_comp(Cpu, Y).

add(mother(X)) causes has(mother(X)) if +(X \== generic).



add(X) causes shopping_cart(X) if true.





add(peripheral(_X)) possible_if true.

add(peripheral(X)) causes has(peripheral(X)) if true.

add(peripheral(X)) causes credit(B1) if +get_value(X,price,P) & ?credit(B) & +(B1 is B + P).

% open_daVinci,ifln(requested(computer(multimedia))).

% open_daVinci,ifln(requested(computer(cad))).

% plan(?credit(X) & ?budget(B) & +(X =< B) after assembly,S),displ(S).

% plan(assembly,S),displ(S).







% ifln(requested(computer(multimedia))).

















































